/*
 *  Copyright 2010 Jesse Long.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */

package net.rumati.sqlblocks;

import java.util.Collections;
import java.util.List;
import net.rumati.sqlblocks.compile.ResultBuilder;

/**
 * The base class, defining the API which an SQL compiler will implement to convert
 * object representations of SQL commands into SQL text.
 * @author Jesse Long
 */
public abstract class BaseSQLCompiler
{
    /**
     * Appends the SQL text for a {@link AndWhereConditionList} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link AndWhereConditionList} is passed to it.
     * @param awcl The {@link AndWhereConditionList} which to convert to SQL.
     * @param result The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendAndWhereConditionListSQL(AndWhereConditionList awcl, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link ColumnExpression} to the given {@link ResultBuilder}.
     * This method is called from {@link #appendExpressionSQL(net.rumati.sqlblocks.Expression, net.rumati.sqlblocks.compile.ResultBuilder)}
     * if a {@link ColumnExpression} if passed to it.
     * @param col The {@link ColumnExpression} to append to the {@link ResultBuilder}.
     * @param result The {@link ResultBuilder} which to append the SQL text to.
     */
    protected abstract void appendColumnExpressionSQL(ColumnExpression col, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link ComparativeWhereCondition} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link ComparativeWhereCondition} is passed to it.
     * @param c The {@link ComparativeWhereCondition} which to convert to SQL.
     * @param r The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendComparativeWhereConditionSQL(ComparativeWhereCondition c, ResultBuilder r);

    /**
     * Appends the SQL text for a {@link ExistsCondition} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link ExistsCondition} is passed to it.
     * @param c The {@link ExistsCondition} which to convert to SQL.
     * @param r The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendExistsConditionSQL(ExistsCondition c, ResultBuilder r);

    /**
     * Appends the SQL text for a {@link Expression} to the given {@link ResultBuilder}.
     * This method will actually call one of the other methods in this class depending on the
     * type of {@link Expression} passed to it.
     * @param e The {@link Expression} to append to the {@link ResultBuilder}.
     * @param r The {@link ResultBuilder} to which to appends the SQL text.
     */
    protected void appendExpressionSQL(Expression e, ResultBuilder r)
    {
        e.compile(this, r);
    }

    /**
     * Appends the SQL text for a {@link IsNotNullCondition} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link IsNotNullCondition} is passed to it.
     * @param c The {@link IsNotNullCondition} which to convert to SQL.
     * @param result The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendIsNotNullWhereConditionSQL(IsNotNullCondition c, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link IsNullCondition} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link IsNullCondition} is passed to it.
     * @param c The {@link IsNullCondition} which to convert to SQL.
     * @param result The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendIsNullWhereConditionSQL(IsNullCondition c, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link MathExpression} to the given {@link ResultBuilder}.
     * This method is called from {@link #appendExpressionSQL(net.rumati.sqlblocks.Expression, net.rumati.sqlblocks.compile.ResultBuilder)}
     * if a {@link MathExpression} if passed to it.
     * @param ex The {@link MathExpression} to append to the {@link ResultBuilder}.
     * @param result The {@link ResultBuilder} which to append the SQL text to.
     */
    protected abstract void appendMathExpressionSQL(MathExpression ex, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link NegatedWhereCondition} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link NegatedWhereCondition} is passed to it.
     * @param nc The {@link NegatedWhereCondition} which to convert to SQL.
     * @param result The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendNegatedWhereConditionSQL(NegatedWhereCondition nc, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link NumericValue} to the given {@link ResultBuilder}.
     * This method is called from {@link #appendExpressionSQL(net.rumati.sqlblocks.Expression, net.rumati.sqlblocks.compile.ResultBuilder)}
     * if a {@link NumericValue} if passed to it.
     * @param v The {@link NumericValue} to append to the {@link ResultBuilder}.
     * @param r The {@link ResultBuilder} which to append the SQL text to.
     */
    protected abstract void appendNumericValueSQL(NumericValue v, ResultBuilder r);

    /**
     * Appends the SQL text for a {@link OrWhereConditionList} to the given {@link ResultBuilder}.
     * This method is called from
     * {@link #appendWhereConditionSQL(net.rumati.sqlblocks.WhereCondition, net.rumati.sqlblocks.compile.ResultBuilder)}
     * when a {@link OrWhereConditionList} is passed to it.
     * @param owc The {@link OrWhereConditionList} which to convert to SQL.
     * @param result The {@link ResultBuilder} which to append the resultant SQL.
     */
    protected abstract void appendOrWhereConditionListSQL(OrWhereConditionList owc, ResultBuilder result);

    /**
     * Appends the SQL text for a {@link ParameterExpression} to the given {@link ResultBuilder}.
     * This method is called from {@link #appendExpressionSQL(net.rumati.sqlblocks.Expression, net.rumati.sqlblocks.compile.ResultBuilder)}
     * if a {@link ParameterExpression} if passed to it.
     * @param ex The {@link MathExpression} to append to the {@link ResultBuilder}.
     * @param res The {@link ResultBuilder} which to append the SQL text to.
     */
    protected abstract void appendParameterExpressionSQL(ParameterExpression ex, ResultBuilder res);

    /**
     * Appends the SQL text for a {@link Relation}.
     * @param rel The relation for which to append the SQL representation.
     * @param r The {@link ResultBuilder} which to append the SQL text to.
     */
    protected void appendRelationDefinition(Relation rel, ResultBuilder r)
    {
        rel.appendDefinition(this, r);
    }

    /**
     * Appends a sub query to the SQL.
     * @param subQuery The sub query to append.
     * @param r The {@link ResultBuilder} to append the SQL to.
     */
    protected abstract void appendSubQuerySQL(SelectQuery subQuery, ResultBuilder r);

    /**
     * Appends the table name to the SQL query.
     * @param t The table to append
     * @param r The {@link ResultBuilder} to append the table name to.
     */
    protected abstract void appendTableName(Table t, ResultBuilder r);

    /**
     * Appends the SQL text for a {@link WhereCondition} to the given {@link ResultBuilder}.
     * This method actually calls one of the other methods in this class depending on the
     * type of {@link WhereCondition} passed to it.
     * @param c The condition for which to append the SQL text.
     * @param r The {@link ResultBuilder} onto which to append the SQL text.
     */
    protected void appendWhereConditionSQL(WhereCondition c, ResultBuilder r)
    {
        c.compile(this, r);
    }

    /**
     * Returns a list a {@link ColumnParameterInfo} objects representing the columns
     * and parameter names for the values to be inserted into the columns.
     * @param cmd The {@link InsertCommand} from which retrieve the column information.
     * @return The list of column to be inserted, and the parameter names for the values
     * of those columns.
     */
    protected List<ColumnParameterInfo> getCommandColumnParameterInfo(InsertCommand cmd)
    {
        return Collections.unmodifiableList(cmd.getColumns());
    }

    /**
     * Returns a list a {@link ColumnParameterInfo} objects representing the columns
     * and parameter names for the columns to be updates
     * @param cmd The {@link UpdateCommand} from which retrieve the column information.
     * @return The list of column to be updates, and the parameter names for the values
     * of those columns.
     */
    protected List<ColumnParameterInfo> getCommandColumnParameterInfo(UpdateCommand cmd)
    {
        return Collections.unmodifiableList(cmd.getColumns());
    }

    /**
     * Returns the default SQL representation of the {@link ComparativeWhereCondition.ComparisonOperator} given.
     * @param op The operator for which to return the SQL token.
     * @return The SQL token as a {@link String}.
     */
    protected String getComparisonOperatorSQL(ComparativeWhereCondition.ComparisonOperator op)
    {
        return op.getSqlToken();
    }

    /**
     * Returns the default SQL token for a {@link MathExpression.MathOperator}.
     * @param op The {@link MathExpression.MathOperator} for which to return a SQL token.
     * @return The SQL token.
     */
    protected String getMathOperatorSQL(MathExpression.MathOperator op)
    {
        return op.getSqlToken();
    }
}
