/*
 *  Copyright 2010 Jesse Long.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */

package net.rumati.sqlblocks;

import net.rumati.sqlblocks.compile.ResultBuilder;

/**
 * Represents a database table or view.
 * @author Jesse Long
 */
public class Table
    extends Relation
{
    /**
     * The schema to which this table belongs. null represents the default schema.
     */
    private final Schema schema;

    /**
     * The name of the table.
     */
    private final String tableName;

    /**
     * Creates a new table object in the default schema.
     * @param tableName the name of the table.
     */
    public Table(String tableName)
    {
        this((Schema)null, tableName);
    }
    
    /**
     * Creates a new table object.
     * @param schemaName The name of the schema in which the table exists.
     * @param tableName The name of the table.
     */
    public Table(String schemaName, String tableName)
    {
        this(new Schema(tableName), tableName);
    }

    /**
     * Creates a new table object.
     * @param schema The schema in which the table exists.
     * @param tableName The name of the table.
     */
    public Table(Schema schema, String tableName)
    {
        this.schema = schema;
        this.tableName = tableName;
    }

    /**
     * Returns the table name.
     * @return The table name.
     */
    public String getTableName()
    {
        return tableName;
    }

    /**
     * Returns the schema in which this table exists, or <code>null</code> if the
     * table is in the default schema.
     * @return The schema in which this table exists, or <code>null</code> if the
     * table is in the default schema.
     */
    public Schema getSchema()
    {
        return schema;
    }

    @Override
    void appendDefinition(BaseSQLCompiler compiler, ResultBuilder r)
    {
        compiler.appendTableName(this, r);
    }

}
